@@ -1,9 +1,8 @@
-//WCC: wrapers for "pblas_level3.f".
-
 #include <R.h>
 #include <Rinternals.h>
 #include "base_global.h"
 
+// PDTRAN
 SEXP R_PDTRAN(SEXP M, SEXP N, SEXP A, SEXP DESCA, SEXP CLDIM, SEXP DESCC)
 {
   SEXP C;
@@ -19,16 +18,15 @@ SEXP R_PDTRAN(SEXP M, SEXP N, SEXP A, SEXP DESCA, SEXP CLDIM, SEXP DESCC)
   
   UNPROTECT(1);
   return(C);
-} /* End of R_PDTRAN(). */
-
+}
 
+// PDGEMM
 SEXP R_PDGEMM(SEXP TRANSA, SEXP TRANSB, SEXP M, SEXP N, SEXP K,
   SEXP A, SEXP DESCA, SEXP B, SEXP DESCB, SEXP CLDIM, SEXP DESCC)
 {
   SEXP C;
   PROTECT(C = allocMatrix(REALSXP, INTEGER(CLDIM)[0], INTEGER(CLDIM)[1]));
   
-/*  const char trans = 'N';*/
   const double alpha = 1.0;
   const double beta = 0.0;
   const int IJ = 1;
@@ -41,5 +39,5 @@ SEXP R_PDGEMM(SEXP TRANSA, SEXP TRANSB, SEXP M, SEXP N, SEXP K,
   
   UNPROTECT(1);
   return(C);
-} /* End of R_PDGEMM(). */
+}
 
